#include "IRremote.hpp"
#define IR_RECEIVE_PIN 9
#define left 6
#define center 5
#define right 4

//all variables used
//motor 1 pins
int AIN1 = 13;
int AIN2 = 8;
int PWMA = 7;
//motor 2 pins
int PWMA2 = 12;
int AIN3 = 10;
int AIN4 = 11;
//distance sensor pins
int trigPin = 3;
int echoPin = 2;
float distance;
 
int speed = 120;
int motorSpeed;
int irbutton;

void setup() {
// Makes all the pins of both motors output the function
  pinMode(AIN1, OUTPUT);
  pinMode(AIN2, OUTPUT);
  pinMode(PWMA, OUTPUT);
  pinMode(AIN3, OUTPUT);
  pinMode(AIN4, OUTPUT);
  pinMode(PWMA2, OUTPUT);
 
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  
  pinMode(left, INPUT);
  pinMode(center, INPUT);
  pinMode(right, INPUT);
  
  Serial.begin(9600);
  IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK);
}

void turnLeft(int &speed) {
  
  Serial.println('right');
  digitalWrite(AIN1, HIGH); //Turns on right wheel forward
  digitalWrite(AIN2, LOW); 
  analogWrite(PWMA, speed);
  digitalWrite(AIN3, LOW);
  digitalWrite(AIN4, HIGH);
  analogWrite(PWMA2, speed);
  //\delay(2000);
}
 
void turnRight(int &speed) { //turns wheels opposite directions to make it go right 
  
  Serial.println('left');
  digitalWrite(AIN1, LOW); 
  digitalWrite(AIN2, HIGH); 
  analogWrite(PWMA, speed);
  digitalWrite(AIN3, HIGH); 
  digitalWrite(AIN4, LOW);//Turns on right wheel forward
  analogWrite(PWMA2, speed);
  //delay(2000);
}  
 
void goForward (int speed) { //writes the ain pins to allow it to go forwards
   
  Serial.println('forward');
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, HIGH);
  analogWrite(PWMA, speed);
  digitalWrite(AIN3, LOW);
  digitalWrite(AIN4, HIGH);
  analogWrite(PWMA2, speed);
  //delay(500);
}
 
void goBackward(int speed) { //reverse of go forward to allow it to go backwards 
  
  Serial.println('backward');
  digitalWrite(AIN1, LOW);
  digitalWrite(AIN2, HIGH);
  analogWrite(PWMA, speed);
  digitalWrite(AIN3, LOW);
  digitalWrite(AIN4, HIGH);
  analogWrite(PWMA2, speed);
  //delay(500);
}
 
void speedUp(int &speed) { //raises the speed incrementally
  speed = speed+1000;  
  analogWrite(PWMA, speed);
  analogWrite(PWMA2, speed);
 Serial.println(speed);
 delay(100);
 
}
 
void speedDown(int &speed) { //lowers the speed incrementally 
  speed = speed-1000;
  analogWrite(PWMA, speed);
  analogWrite(PWMA2, speed);
  Serial.println(speed);
  delay(100);
}
 
void halt(int speed) {

  digitalWrite(AIN1, LOW); //writes them all to low to stop the car
  digitalWrite(AIN2, LOW);
  analogWrite(PWMA, speed);
  digitalWrite(AIN3, LOW);
  digitalWrite(AIN4, LOW);
  analogWrite(PWMA2, speed);
}

void roombaMode(){
  goForward(speed);
  distance = getDistance();  // Check the distance using the ultrasound sensor
  if (distance > 5) {
    goForward(speed);  // Go forward if the distance is greater than 50
  } else if (distance <= 5) {
    turnRight(speed);  // Turn right if the distance is between 36 and 49
  }
  Serial.println(distance);
  delay(100);
  return;
}

void handlekeypress(int irbutton) { //deals with the keypress 
 irbutton=IrReceiver.decodedIRData.command;
 if(irbutton==4){
   turnLeft(speed);
 }
 if(irbutton==6){
   turnRight(speed);
 }
 if(irbutton==5){
   lineSensor();
 }
 if(irbutton==8){
   speedDown(speed);
 }
 if(irbutton==10){
   speedUp(speed);
 }
 if(irbutton==9){
  roombaMode();
  
 }
 if(irbutton==2){
  goForward(speed);
 }
 if(irbutton==1){
  halt(speed);
 }

}
void lineSensor(){
  bool leftV = digitalRead(left); //declaring variable for each sensor
  bool centerV = digitalRead(center);
  bool rightV = digitalRead(right);
  Serial.println(centerV);
  if (leftV == 1 && centerV == 1 && rightV == 1) {
    goForward(speed);
    Serial.println("forward");
  } else if (leftV == 0 && centerV == 0 && rightV == 0) { //conditions for what the line sensor is seeing 
    halt(speed);
  } else if (leftV == 1 && centerV == 0 && rightV == 1) {
    halt(speed);
  } else if (leftV == 0 && centerV == 0 && rightV == 1) {
    turnLeft(speed);
  } else if (leftV == 1 && centerV == 0 && rightV == 0) {
    turnRight(speed);
  } else if (leftV == 0 && centerV == 1 && rightV == 1) {
    turnLeft(speed);
  } else if (leftV == 1 && centerV == 1 && rightV == 0) {
    turnRight(speed);
}
}
float getDistance()
{
	float echoTime; //variable to store the time it takes for a ping to bounce off an object
	float calculatedDistance; //variable to store the distance calculated from the echo time
	//send out an ultrasonic pulse that's 10ms long
	digitalWrite(trigPin, HIGH);
	delayMicroseconds(10);
	digitalWrite(trigPin, LOW);
	echoTime = pulseIn(echoPin, HIGH); //use the pulsein command to see how long it takes for the
	//pulse to bounce back to the sensor
	calculatedDistance = echoTime / 148.0; //calculate the distance of the object that reflected the pulse
	// (half the bounce time multiplied by the speed of sound)
	return calculatedDistance; //send back the distance that was calculated

}

void loop() {
  if(IrReceiver.decode()){ //checks to see if the ir button has been pressed
   int irbutton=IrReceiver.decodedIRData.command; //assigns value to the ir button variable
   Serial.println(irbutton); //prints to check that it read 
   IrReceiver.resume(); //need this to allow it to keep reading 
  }
  handlekeypress(irbutton); //handles the key press 
}
